unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls, Types,
  LazUTF8, LCLIntf, Spin,
  // FreeType
  EasyLazFreeType, LazFreeTypeFontCollection;

type

  { TForm1 }

  TForm1 = class(TForm)
    Edit1: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    PaintBox1: TPaintBox;
    SpinEdit1: TSpinEdit;
    procedure Edit1Change(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
    procedure SpinEdit1Change(Sender: TObject);
  private
    ftFont: TFreeTypeFont;
    FBoundingBox: TRect;
    FLastChar: Cardinal;
//    function GetBoundingBox(AUnicodeChar: Cardinal): TRect;
    function LoadFont(AFontPath: String): TFreeTypeFont;
    procedure UpdateCharAndInfo;

  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

const
  FONT_DIR = 'c:\Windows\Fonts\';
  FONT_NAME = 'Arial.ttf';

{ Sucht das letzte Zeichen des angegebenen Texts. UTF8 wird unterstützt.
  Das gefundene Zeichen wird als Unicode-Codepoint zurückgegeben. }
function FindLastCodePoint(AText: String): Cardinal;
var
  p: PChar;
  charLen, len: Integer;
begin
  if AText = '' then
  begin
    Result := Cardinal(-1);
    exit;
  end;

  p := @AText[1];
  len := Length(AText);
  while len > 0 do
  begin
    Result := UTF8CodePointToUnicode(p, charlen);
    dec(len, charlen);
    inc(P, charLen);
  end;
end;

{ Ermittelt aus der Fontbeschreibung von AFont die Boundingbox für das
  angegebene Unicode-Zeichen. }
function GetBoundingBox(AFont: TFreeTypeFont; ACodePoint: Cardinal): TRect;
var
  glyphIndex: Integer;
  glyph: TFreeTypeGlyph;
  cp: ArrayOfCharPosition;
  p: PChar;
  s: String;
begin
  if (ACodePoint = Cardinal(-1)) or (AFont = nil) then
  begin
    Result := Rect(0, 0, 0, 0);
    exit;
  end;

  glyphIndex := AFont.CharIndex[ACodePoint];
  glyph := AFont.Glyph[glyphIndex];

  s := UnicodeToUTF8(ACodePoint);
  cp := AFont.CharsPosition(s, [ftaLeft, ftaTop]);
  Result := glyph.BoundsWithOffset[cp[0].x, cp[0].yBase];
end;

procedure TForm1.Edit1Change(Sender: TObject);
begin
  UpdateCharAndInfo;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Label1.Caption := '';
  ftFont := LoadFont(FONT_DIR + FONT_NAME);
  ftFont.SizeInPoints := SpinEdit1.Value;
  Paintbox1.Font.Size := SpinEdit1.Value;
  UpdateCharAndInfo;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  ftFont.Free;
end;

{ FreeType-Font mit angegebenem Dateinamen einlesen. Voller Pfad nötig,
  z.B. 'c:\Windows\Fonts\Arial.ttf' }
function TForm1.LoadFont(AFontPath: String): TFreeTypeFont;
var
  FileName, FontFamilyName: string;
begin
  Result := nil;
  FileName := AFontPath;
  if not FileExists(FileName) then
    if (ParamCount > 0) then begin
      FileName := ParamStr(1);
      if not FileExists(FileName) then
        exit;
    end;
  FontFamilyName := FontCollection.AddFile(FileName).Family.FamilyName;
  Result := TFreeTypeFont.Create;
  Result.Name := FontFamilyName;
end;

procedure TForm1.PaintBox1Paint(Sender: TObject);
const
  x = 10;
  y = 10;
var
  s: String;
begin
  if Edit1.Text = '' then
    exit;

  Paintbox1.Canvas.Font.Assign(Paintbox1.Font);
  Paintbox1.Canvas.Brush.Style := bsClear;
  s := UnicodeToUTF8(FLastChar);
  Paintbox1.Canvas.TextOut(x, y, s);

  Paintbox1.Canvas.Pen.Color := clRed;
  with FBoundingBox do
  begin
    Paintbox1.Canvas.Line(x+Left, y+Top, x+Left, y+Bottom);
    Paintbox1.Canvas.Line(x+Right, y+Top, x+Right, y+Bottom);
    Paintbox1.Canvas.Line(x+Left, y+Top, x+Right, y+Top);
    Paintbox1.Canvas.Line(x+Left, y+Bottom, x+Right, y+Bottom);
  end;
end;

procedure TForm1.SpinEdit1Change(Sender: TObject);
begin
  ftFont.SizeInPoints := SpinEdit1.Value;
  Paintbox1.Font.Size := SpinEdit1.Value;
  UpdateCharAndInfo;
end;

{ Extrahiert aus Edit1.Text das letzte Zeichen (FindLastCodePoint), bestimmt
  dafür mit Hilfe des FreeType-Glyphs die Boundingbox, und gibt Zeichen und
  Boundingbox aus. }
procedure TForm1.UpdateCharAndInfo;
begin
  FLastchar := FindLastCodePoint(Edit1.Text);
  FBoundingBox := GetBoundingBox(ftFont, FLastChar);
  with FBoundingBox do
    Label1.Caption := Format('Bounding Box:'#13'  Left=%d'#13'  Top=%d'#13'  Right=%d'#13'  Bottom=%d', [
      Left, Top, Right, Bottom
    ]);
  Paintbox1.Invalidate;
end;


end.

